/*
 * mtftpd - Multi thread server daemon
 * Copyright (C) 2002 Francisco J. Oviedo Alvarez <foviedoa@teleline.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <conf.h>

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_PWD_H
#include <pwd.h>
#endif
#if HAVE_SHADOW_H
#include <shadow.h>
#endif
#if HAVE_STDIO_H
#include <stdio.h>
#endif
#if HAVE_CTYPE_H
#include <ctype.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#if HAVE_TIME_H
#include <time.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_CRYPT_H
#include <crypt.h>
#endif

#include <mtftpd.h>

int mt_auth_check(session_pt ses, const char *pass)
{
	struct passwd *pwd;
#if HAVE_SHADOW_H
	struct spwd *spwd;
#endif

	if( (pwd = getpwnam(ses->name)) == NULL)
		return 0;
	
	if(!(ses->flags & MT_SES_F_ANON)) {
#if HAVE_SHADOW_H
		if(strcmp(pwd->pw_passwd, "x") ||
		(spwd = getspnam(ses->name)) == NULL ||
		spwd->sp_pwdp == NULL)
			return 0;
		if(spwd->sp_pwdp[0] == '@')
			return 0;
		if(spwd->sp_expire > 0 || spwd->sp_max > 0) {
			long today = time(NULL) / (24L * 60L * 60L);

			if( (spwd->sp_expire > 0 && spwd->sp_expire < today) ||
			(spwd->sp_max > 0 && spwd->sp_lstchg > 0 &&
			(spwd->sp_lstchg + spwd->sp_max < today)))
				return 0;
		}	
		if(strcmp(crypt(pass, spwd->sp_pwdp), spwd->sp_pwdp))
			return 0;
#else
		if(strcmp(crypt(pass, pwd->pw_passwd), pwd->pw_passwd))
			return 0;
#endif
	}

	ses->ftpuid = pwd->pw_uid;
	ses->ftpgid = pwd->pw_gid;
	strncpy(ses->rd, pwd->pw_dir, PATH_MAX - 1);
	ses->rd[PATH_MAX - 1] = '\0';
	strcpy(ses->wd, ses->rd);

	return 1;
}

int mt_ftpusers(session_pt ses)
{
	FILE *fh;
	int ret = 1;
	char buf[256];
	char *pbuf;

	fh = fopen(MT_FTPUSERS, "r");
	if(fh == NULL) {
		return ret;
	}

	while( fgets(buf, sizeof(buf) - 1, fh)) {
		buf[sizeof(buf) - 1] = '\0';
		mt_strip_end(buf, "\r\n");

		pbuf = buf;
		while(isspace((int)*pbuf) && *pbuf) pbuf++;

		if(!*pbuf || *pbuf == '#')
			continue;
			
		if(!strcmp(pbuf,ses->name)) {
			ret = 0;
			break;
		}
	}
	fclose(fh);

	return ret;
}
