/*
 * mtftpd - Multi thread server daemon
 * Copyright (C) 2002 Francisco J. Oviedo Alvarez <foviedoa@teleline.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <conf.h>

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STRING
#include <string.h>
#endif

#include <buffer.h>

static int buffer_grow(buffer_pt bp, size_t len)
{
	size_t tt;
	char *tp;

	tt = bp->as + len;
	tt = (tt / bp->is) * bp->is + ((tt % bp->is) ? bp->is:0);

	tp = realloc(bp->dat, tt);
	if(tp == NULL)
		return -1;
	bp->dat = tp;
	bp->ts = tt;
	return 0;
}

buffer_pt buffer_create(size_t is)
{
	buffer_pt tbp;

	tbp = malloc(sizeof(buffer_t));
	if(tbp) {
		tbp->dat = NULL;
		tbp->as = (size_t)0;
		tbp->ts = (size_t)0;
		tbp->is = is;
	}
	return tbp;
}

void buffer_init(buffer_pt bp)
{
	if(bp && bp->dat)
		free(bp->dat);
	bp->dat = NULL;
	bp->as = (size_t)0;
	bp->ts = (size_t)0;
}

void buffer_destroy(buffer_pt bp)
{
	if(bp && bp->dat)
		free(bp->dat);
	if(bp)
		free(bp);
}

int buffer_add(buffer_pt bp, char *sp, size_t len)
{
	if(!bp)
		return -1;

	if( (bp->ts - bp->as) < len)
		if(buffer_grow(bp, len))
			return -1;
	
	memcpy(bp->dat + bp->as, sp, len);
	bp->as += len;

	return 0;
}

char *buffer_getdata(buffer_pt bp)
{
	if(bp)
		return bp->dat;
	else 
		return NULL;
}

size_t buffer_getsize(buffer_pt bp)
{
	if(bp)
		return bp->as;
	else 
		return 0;
}

/* valid only for fixed size data */
void buffer_sort(buffer_pt bp, size_t fs, 
	int(*comp)(const void *, const void *))
{
	qsort(bp->dat, bp->as / fs, fs, comp);
}
