/*
 * mtftpd - Multi thread server daemon
 * Copyright (C) 2002 Francisco Jos Oviedo Alvarez <foviedoa@teleline.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <conf.h>

#if HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#if HAVE_STDIO_H
#include <stdio.h>
#endif
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_SYSLOG_H
#include <syslog.h>
#endif
#if HAVE_ERRNO_H
#include <errno.h>
#endif
#if HAVE_STDARG_H
#include <stdarg.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <mtftpd.h>
#include <log.h>

void log_init(const char *s)
{
#if !MT_DEBUG || MT_WANT_INETD
#ifdef LOG_FTP
	openlog(s, LOG_NDELAY | LOG_PID, LOG_DEBUG | LOG_FTP);
#else
	openlog(s, LOG_NDELAY | LOG_PID, LOG_DEBUG | LOG_DAEMON);
#endif	/* LOG_FTP */
#endif	/* !MT_DEBUG || MT_WANT_INETD */
}

static void log_do(const int err, const int prd, const char *fmt, va_list ap)
{
#define MAXLINE 4096
	int errno_save;
	char buf[MAXLINE];

	errno_save = errno;
	bzero(buf, sizeof(buf));
	vsnprintf(buf, sizeof(buf) - 1, fmt, ap);
	if(err) {
		snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf) - 1,
				": %s", strerror(errno_save));
	}
	strcat(&buf[MIN(sizeof(buf) - 2, strlen(buf))], "\n");
#if MT_DEBUG && !MT_WANT_INETD
	write(STDERR_FILENO, buf, strlen(buf));
#else
	syslog(prd, buf);
#endif
}

void log_ret(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	log_do(1, LOG_WARNING, fmt, ap);
	va_end(ap);
	return;
}

void log_sys(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	log_do(1, LOG_ERR, fmt, ap);
	va_end(ap);
	exit(EXIT_FAILURE);
}

void log_msg(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	log_do(0, LOG_WARNING, fmt, ap);
	va_end(ap);
	return;
}

void log_quit(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	log_do(0, LOG_ERR, fmt, ap);
	va_end(ap);
	exit(EXIT_FAILURE);
}
